==> irssi_0.7.21.orig.tar.gz: irssi-0.7.21/src/gui-gnome/dialog-disconnect.c <==
/*
 dialog-disconnect.c : irssi

    Copyright (C) 1999 Timo Sirainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "irssi.h"

static GtkWidget *clist = NULL;

static void clist_append_server(GtkWidget *clist, SERVER_REC *server)
{
    gchar *titles[4];
    gchar *str;

    g_return_if_fail(clist != NULL);
    g_return_if_fail(server != NULL);

    str = g_strdup_printf("%d", server->port);

    titles[0] = server->tag;
    titles[1] = server->ircnet == NULL ? "" : server->ircnet;
    titles[2] = server->address;
    titles[3] = str;

    gtk_clist_set_row_data(GTK_CLIST(clist), gtk_clist_append(GTK_CLIST(clist), titles), server);

    g_free(str);
}

static void sig_disconnect(GtkWidget *clist)
{
    SERVER_REC *server;
    gint selection;

    g_return_if_fail(clist != NULL);

    selection = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(clist), "selection"));
    if (selection == -1) return;

    server = gtk_clist_get_row_data(GTK_CLIST(clist), selection);
    signal_emit("command disconnect", 3, server->tag, server, cur_channel);
}

static void sig_select_row(GtkWidget *clist, gint row)
{
    g_return_if_fail(clist != NULL);

    gtk_object_set_data(GTK_OBJECT(clist), "selection", GINT_TO_POINTER(row));
}

static void sig_unselect_row(GtkWidget *clist)
{
    g_return_if_fail(clist != NULL);

    gtk_object_set_data(GTK_OBJECT(clist), "selection", GINT_TO_POINTER(-1));
}

static void sig_destroy(GtkWidget **dialog)
{
    g_return_if_fail(dialog != NULL);
    g_return_if_fail(*dialog != NULL);

    gtk_widget_destroy(*dialog);
    *dialog = NULL;
    clist = NULL;
}

static gboolean redraw_servers(void)
{
    GList *tmp;

    if (clist != NULL)
    {
        gtk_clist_freeze(GTK_CLIST(clist));
        gtk_clist_clear(GTK_CLIST(clist));
        for (tmp = g_list_first(servers); tmp != NULL; tmp = tmp->next)
            clist_append_server(clist, tmp->data);
        gtk_clist_thaw(GTK_CLIST(clist));
    }
    return TRUE;
}

static void clist_click_column(GtkCList *clist, gint column)
{
    if (column != clist->sort_column)
	gtk_clist_set_sort_column(clist, column);
    else
    {
	if (clist->sort_type == GTK_SORT_ASCENDING)
	    clist->sort_type = GTK_SORT_DESCENDING;
	else
	    clist->sort_type = GTK_SORT_ASCENDING;
    }

    gtk_clist_sort(clist);
}

void dialog_disconnect(void)
{
    static GtkWidget *dialog = NULL;
    GtkWidget *scrollbox;
    gchar *titles[4];

    if (dialog != NULL)
    {
        /* dialog already open */
        gdk_window_raise(dialog->window);
        return;
    }

    titles[0] = _("Server Tag"); titles[1] = _("IRC Network");
    titles[2] = _("Address"); titles[3] = _("Port");

    dialog = gnome_dialog_new(PACKAGE, _("Disconnect"), GNOME_STOCK_BUTTON_CLOSE, NULL);
    gtk_widget_set_usize(dialog, 350, 250);
    gtk_window_set_policy(GTK_WINDOW(dialog), FALSE, TRUE, FALSE);
    gtk_signal_connect_object(GTK_OBJECT(dialog), "destroy",
                              GTK_SIGNAL_FUNC(sig_destroy), (GtkObject *) &dialog);
    gtk_signal_connect(GTK_OBJECT(dialog), "delete_event",
                       GTK_SIGNAL_FUNC(gtk_widget_destroy), NULL);

    /* Create server clist widget */
    scrollbox = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollbox),
                                   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), scrollbox, TRUE, TRUE, 0);

    clist = gtk_clist_new_with_titles(4, titles);
    gtk_clist_set_column_auto_resize(GTK_CLIST(clist), 0, TRUE);
    gtk_clist_set_column_auto_resize(GTK_CLIST(clist), 1, TRUE);
    gtk_clist_set_column_auto_resize(GTK_CLIST(clist), 2, TRUE);
    gtk_clist_set_column_auto_resize(GTK_CLIST(clist), 3, TRUE);
    gtk_container_add(GTK_CONTAINER(scrollbox), clist);
    gtk_object_set_data(GTK_OBJECT(clist), "selection", GINT_TO_POINTER(-1));
    gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_BROWSE);
    gtk_signal_connect(GTK_OBJECT(clist), "select_row",
                       GTK_SIGNAL_FUNC(sig_select_row), NULL);
    gtk_signal_connect(GTK_OBJECT(clist), "unselect_row",
                       GTK_SIGNAL_FUNC(sig_unselect_row), NULL);
    gtk_signal_connect(GTK_OBJECT(clist), "click_column",
		       GTK_SIGNAL_FUNC(clist_click_column), NULL);

    redraw_servers();
    gtk_clist_select_row(GTK_CLIST(clist), 0, 0);

    gnome_dialog_button_connect_object(GNOME_DIALOG(dialog), 0, GTK_SIGNAL_FUNC(sig_disconnect), GTK_OBJECT(clist));
    gnome_dialog_button_connect_object(GNOME_DIALOG(dialog), 1, GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(dialog));

    /* Make columns resizeable */
    gtk_clist_set_column_resizeable(GTK_CLIST(clist), 0, TRUE);
    gtk_clist_set_column_resizeable(GTK_CLIST(clist), 1, TRUE);
    gtk_clist_set_column_resizeable(GTK_CLIST(clist), 2, TRUE);
    gtk_widget_show_all(dialog);
}

void dialog_disconnect_init(void)
{
    signal_add("server connected", (SIGNAL_FUNC) redraw_servers);
    signal_add("server disconnected", (SIGNAL_FUNC) redraw_servers);
    signal_add("server connect failed", (SIGNAL_FUNC) redraw_servers);
}

void dialog_disconnect_deinit(void)
{
    signal_remove("server connected", (SIGNAL_FUNC) redraw_servers);
    signal_remove("server disconnected", (SIGNAL_FUNC) redraw_servers);
    signal_remove("server connect failed", (SIGNAL_FUNC) redraw_servers);
}

==> irssi_0.7.28.orig.tar.gz: irssi-0.7.28/src/gui-gnome/dialog-disconnect.c <==
/*
 dialog-disconnect.c : irssi

    Copyright (C) 1999 Timo Sirainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "irssi.h"

static GtkWidget *clist = NULL;

static void clist_append_server(GtkWidget *clist, SERVER_REC *server)
{
    gchar *titles[4];

    g_return_if_fail(clist != NULL);
    g_return_if_fail(server != NULL);

    titles[0] = server->tag;
    titles[1] = server->ircnet == NULL ? "" : server->ircnet;
    titles[2] = server->address;
    titles[3] = g_strdup_printf("%d", server->port);

    gtk_clist_set_row_data(GTK_CLIST(clist), gtk_clist_append(GTK_CLIST(clist), titles), server);

    g_free(titles[3]);
}

static void clist_append_reconnect(GtkWidget *clist, RECONNECT_REC *reconnect)
{
    gchar *titles[4];

    g_return_if_fail(clist != NULL);
    g_return_if_fail(reconnect != NULL);

    titles[0] = g_strdup_printf("RECON-%d", reconnect->tag);
    titles[1] = reconnect->ircnet == NULL ? "" : reconnect->ircnet;
    titles[2] = reconnect->address;
    titles[3] = g_strdup_printf("%d", reconnect->port);

    gtk_clist_set_row_data(GTK_CLIST(clist), gtk_clist_append(GTK_CLIST(clist), titles), reconnect);

    g_free(titles[0]);
    g_free(titles[3]);
}

static void sig_disconnect(GtkWidget *clist)
{
    SERVER_REC *server;
    gint selection;

    g_return_if_fail(clist != NULL);

    selection = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(clist), "selection"));
    if (selection == -1) return;

    server = gtk_clist_get_row_data(GTK_CLIST(clist), selection);
    if (g_list_find(reconnects, server) == NULL)
	signal_emit("command disconnect", 3, server->tag, server, cur_channel);
    else
    {
	/* remove reconnection */
	RECONNECT_REC *rec = (RECONNECT_REC *) server;
	gchar *tag;

	tag = g_strdup_printf("RECON-%d", rec->tag);
	signal_emit("command disconnect", 3, tag, NULL, cur_channel);
	g_free(tag);
    }
}

static void sig_select_row(GtkWidget *clist, gint row)
{
    g_return_if_fail(clist != NULL);

    gtk_object_set_data(GTK_OBJECT(clist), "selection", GINT_TO_POINTER(row));
}

static void sig_unselect_row(GtkWidget *clist)
{
    g_return_if_fail(clist != NULL);

    gtk_object_set_data(GTK_OBJECT(clist), "selection", GINT_TO_POINTER(-1));
}

static void sig_destroy(GtkWidget **dialog)
{
    g_return_if_fail(dialog != NULL);
    g_return_if_fail(*dialog != NULL);

    gtk_widget_destroy(*dialog);
    *dialog = NULL;
    clist = NULL;
}

static gboolean redraw_servers(void)
{
    GList *tmp;

    if (clist != NULL)
    {
        gtk_clist_freeze(GTK_CLIST(clist));
        gtk_clist_clear(GTK_CLIST(clist));
        for (tmp = servers; tmp != NULL; tmp = tmp->next)
            clist_append_server(clist, tmp->data);
        for (tmp = lookup_servers; tmp != NULL; tmp = tmp->next)
            clist_append_server(clist, tmp->data);
        for (tmp = reconnects; tmp != NULL; tmp = tmp->next)
            clist_append_reconnect(clist, tmp->data);
	gtk_clist_thaw(GTK_CLIST(clist));

	gtk_object_set_data(GTK_OBJECT(clist), "selection", GINT_TO_POINTER(-1));
	gtk_clist_select_row(GTK_CLIST(clist), 0, 0);
    }
    return TRUE;
}

static void clist_click_column(GtkCList *clist, gint column)
{
    if (column != clist->sort_column)
	gtk_clist_set_sort_column(clist, column);
    else
    {
	if (clist->sort_type == GTK_SORT_ASCENDING)
	    clist->sort_type = GTK_SORT_DESCENDING;
	else
	    clist->sort_type = GTK_SORT_ASCENDING;
    }

    gtk_clist_sort(clist);
}

void dialog_disconnect(void)
{
    static GtkWidget *dialog = NULL;
    GtkWidget *scrollbox;
    gchar *titles[4];

    if (dialog != NULL)
    {
        /* dialog already open */
        gdk_window_raise(dialog->window);
        return;
    }

    titles[0] = _("Server Tag"); titles[1] = _("IRC Network");
    titles[2] = _("Address"); titles[3] = _("Port");

    dialog = gnome_dialog_new(PACKAGE, _("Disconnect"), GNOME_STOCK_BUTTON_CLOSE, NULL);
    gtk_widget_set_usize(dialog, 350, 250);
    gtk_window_set_policy(GTK_WINDOW(dialog), FALSE, TRUE, FALSE);
    gtk_signal_connect_object(GTK_OBJECT(dialog), "destroy",
                              GTK_SIGNAL_FUNC(sig_destroy), (GtkObject *) &dialog);
    gtk_signal_connect(GTK_OBJECT(dialog), "delete_event",
                       GTK_SIGNAL_FUNC(gtk_widget_destroy), NULL);

    /* Create server clist widget */
    scrollbox = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollbox),
                                   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), scrollbox, TRUE, TRUE, 0);

    clist = gtk_clist_new_with_titles(4, titles);
    gtk_clist_set_column_auto_resize(GTK_CLIST(clist), 0, TRUE);
    gtk_clist_set_column_auto_resize(GTK_CLIST(clist), 1, TRUE);
    gtk_clist_set_column_auto_resize(GTK_CLIST(clist), 2, TRUE);
    gtk_clist_set_column_auto_resize(GTK_CLIST(clist), 3, TRUE);
    gtk_container_add(GTK_CONTAINER(scrollbox), clist);
    gtk_object_set_data(GTK_OBJECT(clist), "selection", GINT_TO_POINTER(-1));
    gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_BROWSE);
    gtk_signal_connect(GTK_OBJECT(clist), "select_row",
                       GTK_SIGNAL_FUNC(sig_select_row), NULL);
    gtk_signal_connect(GTK_OBJECT(clist), "unselect_row",
                       GTK_SIGNAL_FUNC(sig_unselect_row), NULL);
    gtk_signal_connect(GTK_OBJECT(clist), "click_column",
		       GTK_SIGNAL_FUNC(clist_click_column), NULL);

    redraw_servers();
    gtk_clist_select_row(GTK_CLIST(clist), 0, 0);

    gnome_dialog_button_connect_object(GNOME_DIALOG(dialog), 0, GTK_SIGNAL_FUNC(sig_disconnect), GTK_OBJECT(clist));
    gnome_dialog_button_connect_object(GNOME_DIALOG(dialog), 1, GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(dialog));

    /* Make columns resizeable */
    gtk_clist_set_column_resizeable(GTK_CLIST(clist), 0, TRUE);
    gtk_clist_set_column_resizeable(GTK_CLIST(clist), 1, TRUE);
    gtk_clist_set_column_resizeable(GTK_CLIST(clist), 2, TRUE);
    gtk_widget_show_all(dialog);
}

void dialog_disconnect_init(void)
{
    signal_add("server looking", (SIGNAL_FUNC) redraw_servers);
    signal_add("server connected", (SIGNAL_FUNC) redraw_servers);
    signal_add("server disconnected", (SIGNAL_FUNC) redraw_servers);
    signal_add("server connect failed", (SIGNAL_FUNC) redraw_servers);
    signal_add("server reconnect new", (SIGNAL_FUNC) redraw_servers);
    signal_add("server reconnect remove", (SIGNAL_FUNC) redraw_servers);
}

void dialog_disconnect_deinit(void)
{
    signal_remove("server looking", (SIGNAL_FUNC) redraw_servers);
    signal_remove("server connected", (SIGNAL_FUNC) redraw_servers);
    signal_remove("server disconnected", (SIGNAL_FUNC) redraw_servers);
    signal_remove("server connect failed", (SIGNAL_FUNC) redraw_servers);
    signal_remove("server reconnect new", (SIGNAL_FUNC) redraw_servers);
    signal_remove("server reconnect remove", (SIGNAL_FUNC) redraw_servers);
}
